package controleurs;

import org.xmldb.api.DatabaseManager;
import org.xmldb.api.base.Collection;
import org.xmldb.api.base.Database;
import org.xmldb.api.base.ResourceIterator;
import org.xmldb.api.base.ResourceSet;
import org.xmldb.api.base.XMLDBException;
import org.xmldb.api.modules.CollectionManagementService;
import org.xmldb.api.modules.XPathQueryService;

import vues.Vue;


/**
 * Classe representant le controleur principal du programme assurant la connexion avec eXist
 * @author erik helsens
 */
public class Controleur {
	/** 
	 * vue associee au controleur
	 */
	private Vue vue;
	/**
	 * service pour effectuer les requetes dans eXist
	 */
	private XPathQueryService service;	
	/**
	 * le projet XML dans eXist
	 */
	private Collection collection;
	
	/**
	 * Constructeur du controleur, initialise la connection avec eXist
	 * @param v la vue associee au controleur
	 * @throws ClassNotFoundException
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 * @throws XMLDBException
	 */
	@SuppressWarnings("unchecked")
	public Controleur(Vue v, String nomCollection) throws ClassNotFoundException, InstantiationException, IllegalAccessException, XMLDBException{
		this.vue = v;

		String driver = "org.exist.xmldb.DatabaseImpl";
		Class cl = Class.forName(driver);
		Database database = (Database)cl.newInstance();
		DatabaseManager.registerDatabase(database);

		String URI = "xmldb:exist://localhost:8080/exist/xmlrpc";
		Collection root = DatabaseManager.getCollection(URI+"/db");
		@SuppressWarnings("unused")
		CollectionManagementService mgtService = (CollectionManagementService)root.getService("CollectionManagementService", "1.0");

		String str_collection = "xmldb:exist://localhost:8080/exist/xmlrpc/db/";
		if(nomCollection != null && nomCollection.compareTo("") != 0)
			str_collection += nomCollection;
		else
			str_collection += "M1MIAGE_HELSENS_ProjetXML";
				
		//Récupération de la collection ProjetXML
		this.collection = DatabaseManager.getCollection(str_collection);		

		service = (XPathQueryService) this.collection.getService("XPathQueryService", "1.0");
		service.setProperty("indent", "yes");
	}

	/**
	 * Effectue une requete sur eXist et l'affiche en console
	 * @param requete la requete a effectuer
	 * @throws XMLDBException
	 */
	public void request(String requete) throws XMLDBException {
		ResourceSet result = service.query(requete);
		ResourceIterator it = result.getIterator();

		while (it.hasMoreResources()){
			System.out.println(it.nextResource().getContent());
		}		
	}

	
	public XPathQueryService getService() {
		return service;
	}

	public Collection getCollection() {
		return collection;
	}

	public Vue getVue() {
		return vue;
	}
}
