package controleurs;

import java.util.Calendar;

import modeles.ContactFull;
import modeles.Evenement;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import vues.VueTools;


/**
 * Clase comportant des outils utilises dans les controleurs
 * @author erik helsens
 */
public class ControleurTools {	
	/**
	 * Cree un element (dom) contact a partir d'un objet ContactFull
	 * @param c l'objet a transformer
	 * @param document le document (dom) de reference
	 * @return un Element (DOM)
	 */
	public static Element createContactElement(ContactFull c, Document document){
		Element res = document.createElement("contact");
		res.setAttribute("id", c.getId());

		Element nom = document.createElement("nom");
		nom.appendChild(document.createTextNode(c.getNom()));
		res.appendChild(nom);

		Element prenom = document.createElement("prenom");		
		prenom.appendChild(document.createTextNode(c.getPrenom()));
		res.appendChild(prenom);

		if(c.getRue() != null) {
			Element rue = document.createElement("rue");
			rue.appendChild(document.createTextNode(c.getRue()));
			res.appendChild(rue);
		}
		if(c.getCodepostal() != null) {
			Element codepostal = document.createElement("codepostal");
			codepostal.appendChild(document.createTextNode(c.getCodepostal()));
			res.appendChild(codepostal);
		}
		if(c.getVille() != null) {
			Element ville = document.createElement("ville");
			ville.appendChild(document.createTextNode(c.getVille()));
			res.appendChild(ville);
		}
		if(c.getPays() != null) {
			Element pays = document.createElement("pays");
			pays.appendChild(document.createTextNode(c.getPays()));
			res.appendChild(pays);
		}
		if (c.getTelephone().size() > 0) {
			Element telephone = document.createElement("telephone");
			Element numero;
			for(int i = 0; i<c.getTelephone().size(); i++){
				numero = document.createElement("numero");
				numero.setAttribute("type", c.getTelephone().get(i).getType());
				numero.appendChild(document.createTextNode(c.getTelephone().get(i).getNumero()));
				telephone.appendChild(numero);
			}
			res.appendChild(telephone);
		}
		if (c.getEmail().size() > 0) {
			Element email;
			for(int i = 0; i<c.getEmail().size(); i++){
				email = document.createElement("email");
				email.appendChild(document.createTextNode(c.getEmail().get(i)));
				res.appendChild(email);
			}
		}
		
		return res;
	}
	
	/**
	 * Retourne le noeud du document de type contact et ayant l'id id
	 * @param document le document DOM ou l'on va chercher
	 * @param id l'id du contact a trouver
	 * @return le noeud correspondant au contact que l'on cherche
	 */
	public static Node getContact(Document document, String id) {
		NodeList list = document.getDocumentElement().getChildNodes();
		Node res = null;
		for(int i = 0; i<list.getLength(); i++) {
			if (list.item(i).getNodeName().equals("contact")) {
				if(list.item(i).getAttributes().item(0).getNodeValue().equals(id)) {
					res = list.item(i);
					break;
				}
			}
		}
		return res;
	}
	
	
	/**
	 * Cree un element (dom) contact a partir d'un objet Evenement
	 * @param c l'objet a transformer
	 * @param document le document (dom) de reference
	 * @return un Element (DOM)
	 */
	public static Element createEvenementElement(Evenement c, Document document){
		Element res = document.createElement("evenement");
		res.setAttribute("id", c.getId());

		Element idContact = document.createElement("idcontact");
		idContact.appendChild(document.createTextNode(c.getIdcontact()));
		res.appendChild(idContact);

		Element date = document.createElement("date");
		date.appendChild(document.createTextNode(VueTools.getDateStringOfGregorianCalendar(c.getDateHeure())));
		res.appendChild(date);
		
		Element heure = document.createElement("heure");
		String heures = c.getDateHeure().get(Calendar.HOUR_OF_DAY)+"";
		if(c.getDateHeure().get(Calendar.HOUR_OF_DAY)<10)
			heures = "0"+heures;
		String minutes = c.getDateHeure().get(Calendar.MINUTE)+"";
		if(c.getDateHeure().get(Calendar.MINUTE)<10)
			minutes = "0"+minutes;
		heure.appendChild(document.createTextNode(heures+":"+minutes));
		res.appendChild(heure);
		
		if(c.getDuree() != null) {
			Element duree = document.createElement("duree");
			duree.appendChild(document.createTextNode(c.getDuree()));
			res.appendChild(duree);
		}
		
		if(c.getObjet() != null) {
			Element objet = document.createElement("objet");
			objet.appendChild(document.createTextNode(c.getObjet()));
			res.appendChild(objet);
		}
		
		if(c.getLieu() != null) {
			Element lieu = document.createElement("lieu");
			lieu.appendChild(document.createTextNode(c.getLieu()));
			res.appendChild(lieu);
		}	

		return res;
	}
	
	
	/**
	 * Retourne le noeud du document de type evenement et ayant l'id id
	 * @param document le document DOM ou l'on va chercher
	 * @param id l'id du contact a trouver
	 * @return le noeud correspondant a l'evenement que l'on cherche
	 */
	public static Node getEvenement(Document document, String id) {
		NodeList list = document.getDocumentElement().getChildNodes();
		Node res = null;
		for(int i = 0; i<list.getLength(); i++) {
			if (list.item(i).getNodeName().equals("evenement")) {
				if(list.item(i).getAttributes().item(0).getNodeValue().equals(id)) {
					res = list.item(i);
					break;
				}
			}
		}
		return res;
	}
	
	/**
	 * Permet d'ajouter un lien vers une feuille de style xsl a un Document
	 * @param doc le Document qu'il faut modifier
	 * @param name le nom du fichier (sans l'extension xsl)
	 *
	public static void addXslStylesheet(Document doc, String name) {
		ProcessingInstruction stylesheet = doc.createProcessingInstruction("xml-stylesheet", "type=\"text/xsl\" href=\""+name+".xsl\"");
		doc.insertBefore(stylesheet, doc.getDocumentElement());
	}
	*/
}
