package modeles;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Classe representant les contacts utilises dans la liste de contacts du controleur (nom, prenom, id)
 * @author erik helsens
 */
@SuppressWarnings("unchecked")
public class Contact implements Comparable {
	/**
	 *  Le nom du contact
	 */
	protected String nom;
	/**
	 * Son prenom
	 */
	protected String prenom;
	/**
	 * Son id
	 */
	protected String id;
	
	/**
	 * Constructeur vide
	 */
	public Contact() {}
	
	/**
	 * Constructeur du contact a partir d'un noeud (DOM)
	 * @param n le noeud (DOM)
	 */
	public Contact (Node n) {
		// récupération des fils de ce noeud
		NodeList listeNodes  = n.getChildNodes();
		// parcours de cette liste
		for(int i = 0; i<listeNodes.getLength(); i++) {
			// récupération du nom
			if (listeNodes.item(i).getNodeName().equals("nom")) {
				this.nom = listeNodes.item(i).getTextContent(); 
			}
			// récupération du prénom
			else if (listeNodes.item(i).getNodeName().equals("prenom")) {
				this.prenom = listeNodes.item(i).getTextContent(); 
			}
		}
		// récupération de l'id
		this.id = n.getAttributes().item(0).getNodeValue();
	}	
	
	/**
	 * Methode issue de l'interface Comparable permettant de donner un critere de tri
	 * @param o le 2e objet a comparer
	 * @return un entier < 0 si this < o, 0 si this == o, > 0 sinon
	 */
	public int compareTo(Object o) {
		Contact c2 = (Contact)o;
		return (this.nom.toUpperCase()+" "+this.prenom).compareTo(c2.nom.toUpperCase()+" "+c2.prenom);
	}
	
	
	public String toString() {
		String res = "";		
		res += nom+" "+prenom+"\n";
		return res;
	}

	public String getNom() {
		return nom;
	}

	public String getPrenom() {
		return prenom;
	}

	public String getId() {
		return id;
	}

	public void setNom(String nom) {
		this.nom = nom;
	}

	public void setPrenom(String prenom) {
		this.prenom = prenom;
	}	
}
