package modeles;

import java.util.ArrayList;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Classe representant les contacts avec tous leurs attributs
 * @author erik helsens
 */
public class ContactFull extends Contact {
	/**
	 * la rue du contact
	 */
	private String rue;
	/**
	 * Son code postal
	 */
	private String codepostal;
	/**
	 * Sa ville
	 */
	private String ville;
	/**
	 * Son pays
	 */
	private String pays;
	/**
	 * Ses numeros de telephone
	 */
	private ArrayList<Numero> telephone;
	/**
	 * Ses emails
	 */
	private ArrayList<String> email;

	/**
	 * Constructeur du contact a partir d'un noeud (DOM)
	 * @param n le noeud (DOM)
	 */
	public ContactFull(Node n) {
		super(n);
		NodeList listeNodes, listeNumeros; 
		listeNodes  = n.getChildNodes();
		this.telephone = new ArrayList<Numero>();
		this.email = new ArrayList<String>();
		for(int i = 0; i<listeNodes.getLength(); i++) {
			if (listeNodes.item(i).getNodeName().equals("rue")) {
				this.rue = listeNodes.item(i).getTextContent();
			}
			else if (listeNodes.item(i).getNodeName().equals("codepostal")) {
				this.codepostal = listeNodes.item(i).getTextContent();
			}
			else if (listeNodes.item(i).getNodeName().equals("ville")) {
				this.ville = listeNodes.item(i).getTextContent();
			}
			else if (listeNodes.item(i).getNodeName().equals("pays")) {
				this.pays = listeNodes.item(i).getTextContent();
			}
			else if (listeNodes.item(i).getNodeName().equals("email")) {
				this.email.add(listeNodes.item(i).getTextContent()); 
			}
			else if (listeNodes.item(i).getNodeName().equals("telephone")) {
				listeNumeros = listeNodes.item(i).getChildNodes();
				for(int j=0; j<listeNumeros.getLength(); j++) {
					if (listeNumeros.item(j).getNodeName().equals("numero"))
						this.telephone.add(new Numero(listeNumeros.item(j).getAttributes().item(0).getNodeValue(),listeNumeros.item(j).getTextContent()));
				}
			}
		}
	}

	/**
	 * Constructeur vide (utilise dans la creation de contact)<br/>
	 * Un id est genere
	 */
	public ContactFull() {
		super();
		this.telephone = new ArrayList<Numero>(); 
		this.email = new ArrayList<String>();
		this.id = ListeContact.generateNewID();
	}
	
	/**
	 * Constructeur a partir d'un id (utilise dans l'edition de contact)
	 * @param id l'id du contact
	 */
	public ContactFull(String id) {
		super();
		this.telephone = new ArrayList<Numero>(); 
		this.email = new ArrayList<String>();
		this.id = id;
	}

	public String toString() {
		String res = "<html>";
		res += "<h1>"+this.nom.toUpperCase()+" "; 
		res += this.prenom+"</h1>";
		if(this.rue != null)
			res += this.rue+"<br/>";
		if(this.codepostal != null)
			res += this.codepostal+" ";
		if(this.ville != null)
			res += this.ville+"<br/>";
		if(this.pays != null)
			res += this.pays+"<br/>";
		
		if(this.telephone.size() > 0) {
			for(int i=0; i<telephone.size(); i++){
				res += "<b>"+this.telephone.get(i).getType()+":</b> "+this.telephone.get(i).getNumero()+"<br/>";
			}
		}
		if(this.email.size() > 0) {
			for(int i=0; i<email.size(); i++){
				res += this.email.get(i)+"<br/>";
			}
		}					
		return res+"</html>";
	}

	public String getRue() {
		return rue;
	}

	public void setRue(String rue) {
		this.rue = rue;
	}

	public String getCodepostal() {
		return codepostal;
	}

	public void setCodepostal(String codepostal) {
		this.codepostal = codepostal;
	}

	public String getVille() {
		return ville;
	}

	public void setVille(String ville) {
		this.ville = ville;
	}

	public String getPays() {
		return pays;
	}

	public void setPays(String pays) {
		this.pays = pays;
	}

	public ArrayList<Numero> getTelephone() {
		return telephone;
	}

	public void setTelephone(ArrayList<Numero> telephone) {
		this.telephone = telephone;
	}

	public ArrayList<String> getEmail() {
		return email;
	}

	public void setEmail(ArrayList<String> email) {
		this.email = email;
	};

	public void addNumero(String type, String num) {
		this.telephone.add(new Numero(type, num));
	}

	public void addEmail(String email) {
		this.email.add(email);
	};

}
