package modeles;

import java.util.Calendar;
import java.util.GregorianCalendar;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Classe representant les evenements de l'agenda
 * @author erik helsens 
 */

@SuppressWarnings("unchecked")
public class Evenement implements Comparable {
	/**
	 * son id
	 */
	private String id;
	/**
	 * l'id du contact associe
	 */
	private String idcontact;
	/**
	 * date et heure de l'evenement
	 */
	private GregorianCalendar dateHeure;
	/**
	 * duree (non numerique)
	 */
	private String duree;
	/**
	 * objet de l'evenement
	 */
	private String objet;
	/**
	 * lieu de l'evenement
	 */
	private String lieu;
	
	/**
	 * Constructeur de l'evenement a partir d'un noeud (DOM)
	 * @param n le noeud (DOM)
	 */
	public Evenement(Node n) {
		NodeList listeNodes = n.getChildNodes();
		// recup de l'id
		this.id = n.getAttributes().item(0).getNodeValue();
		for(int i = 0; i<listeNodes.getLength(); i++) {
			if (listeNodes.item(i).getNodeName().equals("idcontact")) {
				this.idcontact = listeNodes.item(i).getTextContent(); 
			}
			else if (listeNodes.item(i).getNodeName().equals("date")) {
				try {
					String strDate = listeNodes.item(i).getTextContent();
					int day = Integer.parseInt(strDate.substring(0,2));
					int month = Integer.parseInt(strDate.substring(3,5));
					int year = Integer.parseInt(strDate.substring(6,10));
					this.dateHeure = new GregorianCalendar(year, month-1, day);
				}
				catch (Exception e){
					System.err.println("date parsing error (evenement id="+this.id+")");
					System.exit(1);
				}
			}
			else if (listeNodes.item(i).getNodeName().equals("heure")) {
				try {
					String strHeure = listeNodes.item(i).getTextContent();
					int heure = Integer.parseInt(strHeure.substring(0,2));
					int minutes = Integer.parseInt(strHeure.substring(3,5));
					this.dateHeure.set(GregorianCalendar.HOUR, heure);
					this.dateHeure.set(GregorianCalendar.MINUTE, minutes);
				}
				catch (Exception e){
					System.err.println("hour parsing error (evenement id="+this.id+")");
					System.exit(1);
				}
				
			}
			else if (listeNodes.item(i).getNodeName().equals("duree")) {
				this.duree = listeNodes.item(i).getTextContent();
			}
			else if (listeNodes.item(i).getNodeName().equals("objet")) {
				this.objet = listeNodes.item(i).getTextContent();
			}
			else if (listeNodes.item(i).getNodeName().equals("lieu")) {
				this.lieu = listeNodes.item(i).getTextContent();
			}
		}
	}

	/**
	 * Constructeur vide (utilise dans la creation d'evenement)<br/>
	 * Un id est genere
	 */
	public Evenement() {
		this.id = ListeEvenement.generateNewID();
	}
	
	/**
	 * Constructeur a partir d'un id (utilise dans l'edition d'evenement)
	 * @param id l'id de l'evenement
	 */
	public Evenement(String id) {
		this.id = id;
	}
	
	/**
	 * Methode issue de l'interface Comparable permettant de donner un critere de tri
	 * @param o le 2e objet a comparer
	 * @return un entier < 0 si this < o, 0 si this == o, > 0 sinon
	 */
	public int compareTo(Object o) {
		Evenement e2 = (Evenement)o;
		return this.dateHeure.compareTo(e2.getDateHeure());
	}

	public String toString() {
		String res = "";
		if (id != null)		res += "id: "+id+"\n";
		if (idcontact != null)		res += "idcontact: "+idcontact+"\n";
		if (dateHeure != null) {
			res += "dateHeure: "+dateHeure.get(Calendar.DATE)+"/"+dateHeure.get(Calendar.MONTH)+1+"/"+dateHeure.get(Calendar.YEAR)
				+" "+dateHeure.get(Calendar.HOUR)+":"+dateHeure.get(Calendar.MINUTE)+"\n";
		}
		if (duree != null)		res += "duree: "+duree+"\n";
		if (objet != null)		res += "objet: "+objet+"\n";
		if (lieu != null)		res += "lieu: "+lieu+"\n";
		return res;
	}

	public String getIdcontact() {
		return idcontact;
	}

	public void setIdcontact(String idcontact) {
		this.idcontact = idcontact;
	}

	public GregorianCalendar getDateHeure() {
		return dateHeure;
	}

	public void setDateHeure(GregorianCalendar dateHeure) {
		this.dateHeure = dateHeure;
	}

	public String getDuree() {
		return duree;
	}

	public void setDuree(String duree) {
		this.duree = duree;
	}

	public String getObjet() {
		return objet;
	}

	public void setObjet(String objet) {
		this.objet = objet;
	}

	public String getLieu() {
		return lieu;
	}

	public void setLieu(String lieu) {
		this.lieu = lieu;
	}

	public String getId() {
		return id;
	}
}
