package modeles;

import java.io.IOException;
import java.util.ArrayList;

import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

public class ListeEvenement extends ArrayList<Evenement> {
	private static final long serialVersionUID = -779932112614402481L;
	
	/**
	 * l'id max des evenements
	 */
	private static int ID_MAX;

	/**
	 * Constructeur a partir d'un document DOM qui le resultat de la requete sur les contacts
	 * @param doc le document DOM
	 * @throws DOMException
	 * @throws SAXException
	 * @throws IOException
	 * @throws NumberFormatException
	 */
	public ListeEvenement(Document doc) throws DOMException, SAXException, IOException, NumberFormatException {
		// on récupère la liste des contacts
		NodeList listeTmp = doc.getFirstChild().getChildNodes();
		// parcours et ajout dans la liste courante
		for(int i = 0; i<listeTmp.getLength(); i++) {
			if (listeTmp.item(i).getNodeName().equals("evenement"))
				add(new Evenement(listeTmp.item(i)));
		}
		// ajustement de la longueur de la liste
		this.trimToSize();
		// récupération de l'id maximum
		ID_MAX = this.findMaxId();
	}

	/**
	 * Cherche l'id maximum de la liste
	 * @return l'id maximum de la liste
	 */
	private int findMaxId() {
		int max = -1;
		for(int i = 0; i<this.size(); i++){
			if (Integer.parseInt(get(i).getId()) > max)
				max = Integer.parseInt(get(i).getId());
		}
		return max;
	}
	
	public static int getID_MAX() {
		return ID_MAX;
	}
	
	/**
	 * Genere un nouvel id a partir de l'id MAX actuel
	 * @return un nouvel id a partir de l'id MAX actuel
	 */
	public static String generateNewID() {
		return new Integer(ID_MAX+1).toString();
	}
	
	public String toString(){
		String res = "";
		for(int i = 0; i<this.size(); i++){
			res += this.get(i)+"\n";
		}
		return res;
	}
}
