package modeles;

import java.util.Calendar;
import java.util.GregorianCalendar;

import vues.VueTools;

/**
 * Classe utilisee pour la gestion du calendrier
 * @author erik helsens
 */
public class SemaineCourante {
	/**
	 * la date du lundi courant
	 */
	private GregorianCalendar dateLundi;
	/**
	 * la date du jour
	 */
	private GregorianCalendar dateJour;

	/**
	 * Constructeur qui initialise le lundi de la semaine courante en fonction du jour courant.
	 */
	public SemaineCourante(){
		// récupération de la date courante
		GregorianCalendar date = (GregorianCalendar)GregorianCalendar.getInstance();		
		this.dateLundi = new GregorianCalendar(date.get(Calendar.YEAR),date.get(Calendar.MONTH),date.get(Calendar.DAY_OF_MONTH)); 
		this.dateJour = new GregorianCalendar(date.get(Calendar.YEAR),date.get(Calendar.MONTH),date.get(Calendar.DAY_OF_MONTH));
		// attention premier jour de la semaine = dimanche
		int jourSemaine = this.dateLundi.get(Calendar.DAY_OF_WEEK);
		// en fonction du jour de la semaine, initialisation du lundi
		switch (jourSemaine){
		case Calendar.MONDAY:
			break;
		case Calendar.TUESDAY:
			this.dateLundi.add(Calendar.DAY_OF_YEAR,-1);
			break;
		case Calendar.WEDNESDAY:
			this.dateLundi.add(Calendar.DAY_OF_YEAR,-2);
			break;
		case Calendar.THURSDAY:
			this.dateLundi.add(Calendar.DAY_OF_YEAR,-3);
			break;			
		case Calendar.FRIDAY:
			this.dateLundi.add(Calendar.DAY_OF_YEAR,-4);
			break;
		case Calendar.SATURDAY:
			this.dateLundi.add(Calendar.DAY_OF_YEAR,-5);
			break;			
		case Calendar.SUNDAY:
			this.dateLundi.add(Calendar.DAY_OF_YEAR,-6);
			break;			
		}
	}


	public GregorianCalendar getDateLundi() {
		return dateLundi;
	}

	/**
	 * Retourne la date de la semaine courante associee au jour passe en parametre
	 * @param CalendarDayValue le jour dont on veut la date
	 * @return la date de la semaine courante associee au jour passe en parametre
	 */
	public GregorianCalendar getCurrentDateByCalendarDayValue(int CalendarDayValue) {
		// création d'une nouvelle date à partir du lundi courant
		GregorianCalendar date = new GregorianCalendar(this.dateLundi.get(Calendar.YEAR),this.dateLundi.get(Calendar.MONTH),this.dateLundi.get(Calendar.DAY_OF_MONTH));
		// en fonction du jour on augment cette date ou pas
		switch (CalendarDayValue){
		case Calendar.MONDAY:
			break;
		case Calendar.TUESDAY:
			date.add(Calendar.DAY_OF_YEAR,1);
			break;
		case Calendar.WEDNESDAY:
			date.add(Calendar.DAY_OF_YEAR,2);
			break;
		case Calendar.THURSDAY:
			date.add(Calendar.DAY_OF_YEAR,3);
			break;			
		case Calendar.FRIDAY:
			date.add(Calendar.DAY_OF_YEAR,4);
			break;
		case Calendar.SATURDAY:
			date.add(Calendar.DAY_OF_YEAR,5);
			break;			
		case Calendar.SUNDAY:
			date.add(Calendar.DAY_OF_YEAR,6);
			break;			
		}
		return date;
	}
	
	/**
	 * Methode utilise dans le controleur pour effectuer la requete sur les dates de la semaine courante
	 * @return un tableau de chaines de caracteres correspondant aux jours de la semaine
	 */
	public String[] getWeekStringDates() {
		String[] res = new String[7];
		GregorianCalendar date = new GregorianCalendar(this.dateLundi.get(Calendar.YEAR),this.dateLundi.get(Calendar.MONTH),this.dateLundi.get(Calendar.DAY_OF_MONTH));
		res[0] = VueTools.getDateStringOfGregorianCalendar(date);
		for(int i = 1; i<res.length; i++){
			date.add(Calendar.DAY_OF_YEAR, 1);
			res[i] = VueTools.getDateStringOfGregorianCalendar(date);
		}
		return res;
	}
	
	/**
	 * Passe a la semaine suivante
	 */
	public void weekAfter() {
		this.dateLundi.add(Calendar.DAY_OF_YEAR, 7);
	}
	
	/**
	 * Passe a la semaine precedente
	 */
	public void previousWeek(){
		this.dateLundi.add(Calendar.DAY_OF_YEAR, -7);
	}
	
	/**
	 * Passe au mois suivant
	 */
	public void monthAfter() {
		this.dateLundi.add(Calendar.DAY_OF_YEAR, 28);
	}
	
	/**
	 * Passe au mois precedent
	 */
	public void previousMonth(){
		this.dateLundi.add(Calendar.DAY_OF_YEAR, -28);
	}
	

	public String toString() {
		return this.dateLundi.toString();
	}


	public GregorianCalendar getDateJour() {
		return dateJour;
	}
	
	public int compareDateJour(GregorianCalendar d2){
		int res = -1;
		if(this.dateJour.get(Calendar.DATE)==d2.get(Calendar.DATE) 
		    && this.dateJour.get(Calendar.MONTH)==d2.get(Calendar.MONTH)
		    && this.dateJour.get(Calendar.YEAR)==d2.get(Calendar.YEAR))
			res = 0;
		return res;
	}
}
