package vues;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;

import modeles.ListeContact;
import modeles.ListeEvenement;

import org.xml.sax.SAXException;
import org.xmldb.api.base.XMLDBException;

import controleurs.Controleur;
import controleurs.ControleurAgenda;


/**
 * Classe representant la vue de l'agenda
 * @author erik helsens
 */
public class PanelAgenda extends JPanel {
	private static final long serialVersionUID = -8655074747144465060L;

	/**
	 * le controleur du programme
	 */
	private ControleurAgenda controleurAgenda;
	/**
	 * label affichant la plage de dates de la semaine courante
	 */
	private JLabel labelPanelHaut;
	/**
	 * tableau des labels de date des jours de la semaine
	 */
	private JLabel[] tabLabel;
	/**
	 * tableau des listes d'evenements de la semaine
	 */
	private JList[] tabList;
	/**
	 * composant ou on affiche les details d'un evenement (lorsqu'il est selectionne)
	 */
	private JTextPane textEvenement;
	/**
	 * vue d'ajout / edition d'un evenement
	 */
	private VueEvenement vueEvenement;
	/**
	 * combo liste des contacts utilisee dans le filtrage des evenements
	 */
	private JComboBox comboContacts;
	/**
	 * liste des contacts
	 */
	private ListeContact contacts;

	/**
	 * Constructeur
	 * @param c le controleur du programme
	 * @throws IOException 
	 * @throws SAXException 
	 * @throws XMLDBException 
	 */
	public PanelAgenda(Controleur c) throws XMLDBException, SAXException, IOException {
		// création du controleur de l'agenda à partir du controleur du programme
		this.controleurAgenda = new ControleurAgenda(this,c.getService(),c.getCollection(),c);
		

		/* Panel haut */
		this.labelPanelHaut = new JLabel("",JLabel.CENTER);

		JButton bReculerMois = new JButton("<<");
		bReculerMois.setToolTipText("Reculer d'un mois");
		bReculerMois.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				try {
					controleurAgenda.previousMonth();
				} catch (XMLDBException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				} catch (SAXException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				} catch (IOException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}				
			}			
		});
		
		JButton bReculerSemaine = new JButton("<");
		bReculerSemaine.setToolTipText("Reculer d'une semaine");
		bReculerSemaine.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				try {
					controleurAgenda.previousWeek();
				} catch (XMLDBException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				} catch (SAXException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				} catch (IOException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}				
			}			
		});

		JPanel panelHautGauche = new JPanel();
		panelHautGauche .add(bReculerMois);
		panelHautGauche .add(bReculerSemaine);
		
		

		JButton bAvancerSemaine = new JButton(">");
		bAvancerSemaine.setToolTipText("Avancer d'une semaine");
		bAvancerSemaine.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				try {
					controleurAgenda.weekAfter();
				} catch (XMLDBException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				} catch (SAXException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				} catch (IOException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}				
			}			
		});
		
		JButton bAvancerMois = new JButton(">>");
		bAvancerMois.setToolTipText("Avancer d'un mois");
		bAvancerMois.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				try {
					controleurAgenda.monthAfter();
				} catch (XMLDBException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				} catch (SAXException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				} catch (IOException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}				
			}			
		});

		JPanel panelHautDroit = new JPanel();
		panelHautDroit.add(bAvancerSemaine);
		panelHautDroit.add(bAvancerMois);
		
		JPanel panelHaut = new JPanel(new BorderLayout());
		panelHaut.add(labelPanelHaut,BorderLayout.CENTER);
		panelHaut.add(panelHautGauche,BorderLayout.WEST);
		panelHaut.add(panelHautDroit,BorderLayout.EAST);


		/* Panel milieu */
		JPanel panelMilieu = new JPanel(new GridLayout(2,4));

		this.tabLabel = new JLabel[7];

		JPanel[] tabPanel = new JPanel[7];		
		this.tabList = new JList[7];
		JScrollPane[] tabScroll = new JScrollPane[7];

		for(int i = 0; i<tabPanel.length; i++) {
			this.tabLabel[i] = new JLabel("");
			tabPanel[i] = new JPanel(new BorderLayout());
			this.tabList[i] = new JList();
			this.tabList[i].addMouseListener(new AfficheEvenementMouseListener());
			this.tabList[i].addKeyListener(new AfficheEvenementKeyListener());
			tabScroll[i] = new JScrollPane(this.tabList[i]);
			tabPanel[i].add(this.tabLabel[i],BorderLayout.NORTH);
			tabPanel[i].add(tabScroll[i],BorderLayout.CENTER);
			tabPanel[i].add(new JLabel("   "),BorderLayout.SOUTH);
			panelMilieu.add(tabPanel[i]);
		}

		this.comboContacts = new JComboBox();
		this.comboContacts.setToolTipText("Filtrage des événements");
		this.setContacts();
		this.comboContacts.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				if(comboContacts.getSelectedIndex() > 0)
					controleurAgenda.setIdContactFiltre(contacts.get(comboContacts.getSelectedIndex()-1).getId());
				else
					controleurAgenda.setIdContactFiltre(null);

				try {
					controleurAgenda.requestCurrentWeekEvenements();
				} catch (XMLDBException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				} catch (SAXException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				} catch (IOException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
			}
		});	

		// ajout de la combo contact dans le panel
		JPanel panelContacts = new JPanel(new GridLayout(5,1));
		panelContacts.add(new JLabel(""));
		panelContacts.add(new JLabel(""));
		panelContacts.add(new JLabel(""));
		panelContacts.add(new JLabel("Filtre par contact"));
		panelContacts.add(this.comboContacts);
		JPanel panelMilieuContacts = new JPanel(new BorderLayout());
		panelMilieuContacts.add(new JLabel(" "),BorderLayout.WEST);
		panelMilieuContacts.add(new JLabel(" "),BorderLayout.SOUTH);
		panelMilieuContacts.add(panelContacts,BorderLayout.CENTER);
		panelMilieu.add(panelMilieuContacts);


		/* Panel bas */
		// Boutton Actualiser
		JButton bActualiser = new JButton("Actualiser");
		bActualiser.setToolTipText("Actualiser l'affichage des événements");
		bActualiser.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				try {
					controleurAgenda.requestCurrentWeekEvenements();
					clearTextEvenement();
					setContacts();
				} catch (XMLDBException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				} catch (SAXException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				} catch (IOException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
			}			
		});

		// Boutton Ajouter
		JButton bAjouter = new JButton("Ajouter");
		bAjouter.setToolTipText("Ajouter un événement");
		bAjouter.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				vueEvenement = new VueEvenement(controleurAgenda,true,"");	
			}			
		});

		// Boutton Editer
		JButton bEditer = new JButton("Editer");
		bEditer.setToolTipText("Editer l'événement sélectionné");
		bEditer.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				if (controleurAgenda.getEvenementAffiche() != null) {
					vueEvenement = new VueEvenement(controleurAgenda,false, controleurAgenda.getEvenementAffiche().getId());
					vueEvenement.setEvenement(controleurAgenda.getEvenementAffiche());
				}
			}			
		});

		// Boutton Supprimer
		JButton bSupprimer = new JButton("Supprimer");
		bSupprimer.setToolTipText("Supprimer l'événement sélectionné");
		bSupprimer.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				if (controleurAgenda.getEvenementAffiche() != null) {
					if (JOptionPane.showConfirmDialog(null, "Êtes-vous sur de vouloir supprimer cet événement?", 
							"Suppression d'un événement...", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE) 
							== JOptionPane.YES_OPTION) {
						try {
							controleurAgenda.removeEvenement(controleurAgenda.getEvenementAffiche().getId());
						} catch (XMLDBException e1) {
							// TODO Auto-generated catch block
							e1.printStackTrace();
						} catch (SAXException e1) {
							// TODO Auto-generated catch block
							e1.printStackTrace();
						} catch (IOException e1) {
							// TODO Auto-generated catch block
							e1.printStackTrace();
						}
					}
				}
			}			
		});

		/* panel des bouttons du bas */
		JPanel panelBouttons = new JPanel(new GridLayout(1,4));
		panelBouttons.add(bActualiser);
		panelBouttons.add(bAjouter);
		panelBouttons.add(bEditer);
		panelBouttons.add(bSupprimer);

		// texte événement
		this.textEvenement = new JTextPane();
		this.textEvenement.setEditable(false);
		this.textEvenement.setContentType("text/html");
		this.textEvenement.setPreferredSize(new Dimension(60, 50));
		JScrollPane scrollTextEvenement = new JScrollPane(this.textEvenement);

		JPanel panelBas = new JPanel(new BorderLayout());
		panelBas.add(new JLabel("Événement"),BorderLayout.NORTH);
		panelBas.add(scrollTextEvenement,BorderLayout.CENTER);
		panelBas.add(panelBouttons,BorderLayout.SOUTH);

		// ajout des différents panels
		this.setLayout(new BorderLayout());
		this.add(panelHaut, BorderLayout.NORTH);		
		this.add(panelMilieu, BorderLayout.CENTER);
		this.add(panelBas, BorderLayout.SOUTH);

		// mise à jour des dates
		this.updateDates();

		// recherche effectuée sur tous les événements
		this.controleurAgenda.requestCurrentWeekEvenements();
	}


	/**
	 * Met a jour l'affichage en cas de changement de semaine
	 * @param list la liste des evenements de la semaine
	 * @throws XMLDBException
	 * @throws SAXException
	 * @throws IOException
	 */
	public void changeWeek(ListeEvenement list) throws XMLDBException, SAXException, IOException {
		// MAJ des dates
		updateDates();
		// MAJ des listes
		setTabList(list);
		// on efface l'affichage de l'evenement en cours
		this.clearTextEvenement();
	}

	/**
	 * Mise a jour des dates en fonction de la semaine courante
	 */
	private void updateDates() {
		this.labelPanelHaut.setText("Semaine du "+VueTools.getDateStringOfGregorianCalendar(controleurAgenda.getDateLundiSemaineCourante())+" au "
				+VueTools.getDateStringOfGregorianCalendar(controleurAgenda.getCurrentDateByCalendarDayValue(Calendar.SUNDAY)));
		
		this.tabLabel[0].setText("Lundi "+VueTools.getShortDateStringOfGregorianCalendar(controleurAgenda.getCurrentDateByCalendarDayValue(Calendar.MONDAY)));
		if(controleurAgenda.compareDateJour(controleurAgenda.getCurrentDateByCalendarDayValue(Calendar.MONDAY))==0)
			this.tabLabel[0].setForeground(Color.BLUE);
		else
			this.tabLabel[0].setForeground(Color.BLACK);
		
		this.tabLabel[1].setText("Mardi "+VueTools.getShortDateStringOfGregorianCalendar(controleurAgenda.getCurrentDateByCalendarDayValue(Calendar.TUESDAY)));
		if(controleurAgenda.compareDateJour(controleurAgenda.getCurrentDateByCalendarDayValue(Calendar.TUESDAY))==0)
			this.tabLabel[1].setForeground(Color.BLUE);
		else
			this.tabLabel[1].setForeground(Color.BLACK);
		
		this.tabLabel[2].setText("Mercredi "+VueTools.getShortDateStringOfGregorianCalendar(controleurAgenda.getCurrentDateByCalendarDayValue(Calendar.WEDNESDAY)));
		if(controleurAgenda.compareDateJour(controleurAgenda.getCurrentDateByCalendarDayValue(Calendar.WEDNESDAY))==0)
			this.tabLabel[2].setForeground(Color.BLUE);
		else
			this.tabLabel[2].setForeground(Color.BLACK);
		
		this.tabLabel[3].setText("Jeudi "+VueTools.getShortDateStringOfGregorianCalendar(controleurAgenda.getCurrentDateByCalendarDayValue(Calendar.THURSDAY)));
		if(controleurAgenda.compareDateJour(controleurAgenda.getCurrentDateByCalendarDayValue(Calendar.THURSDAY))==0)
			this.tabLabel[3].setForeground(Color.BLUE);
		else
			this.tabLabel[3].setForeground(Color.BLACK);
		
		this.tabLabel[4].setText("Vendredi "+VueTools.getShortDateStringOfGregorianCalendar(controleurAgenda.getCurrentDateByCalendarDayValue(Calendar.FRIDAY)));
		if(controleurAgenda.compareDateJour(controleurAgenda.getCurrentDateByCalendarDayValue(Calendar.FRIDAY))==0)
			this.tabLabel[4].setForeground(Color.BLUE);
		else
			this.tabLabel[4].setForeground(Color.BLACK);
		
		this.tabLabel[5].setText("Samedi "+VueTools.getShortDateStringOfGregorianCalendar(controleurAgenda.getCurrentDateByCalendarDayValue(Calendar.SATURDAY)));
		if(controleurAgenda.compareDateJour(controleurAgenda.getCurrentDateByCalendarDayValue(Calendar.SATURDAY))==0)
			this.tabLabel[5].setForeground(Color.BLUE);
		else
			this.tabLabel[5].setForeground(Color.BLACK);
		
		this.tabLabel[6].setText("Dimanche "+VueTools.getShortDateStringOfGregorianCalendar(controleurAgenda.getCurrentDateByCalendarDayValue(Calendar.SUNDAY)));
		if(controleurAgenda.compareDateJour(controleurAgenda.getCurrentDateByCalendarDayValue(Calendar.SUNDAY))==0)
			this.tabLabel[6].setForeground(Color.BLUE);
		else
			this.tabLabel[6].setForeground(Color.BLACK);
	}

	/**
	 * Efface l'affichage de l'evenement
	 */
	public void clearTextEvenement() {
		this.textEvenement.setText("");
		this.controleurAgenda.setEvenementAffiche(null);
	}

	/**
	 * Met les evenements de list dans les differentes listes
	 * @param list la liste d'evenement
	 * @throws XMLDBException
	 * @throws SAXException
	 * @throws IOException
	 */
	public void setTabList(ListeEvenement list) throws XMLDBException, SAXException, IOException {
		ArrayList<String> lundi = new ArrayList<String>();
		ArrayList<String> mardi = new ArrayList<String>();
		ArrayList<String> mercredi = new ArrayList<String>();
		ArrayList<String> jeudi = new ArrayList<String>();
		ArrayList<String> vendredi = new ArrayList<String>();
		ArrayList<String> samedi = new ArrayList<String>();
		ArrayList<String> dimanche = new ArrayList<String>();
		GregorianCalendar dateLundiWeakAfter = new GregorianCalendar(controleurAgenda.getDateLundiSemaineCourante().get(Calendar.YEAR),controleurAgenda.getDateLundiSemaineCourante().get(Calendar.MONTH),controleurAgenda.getDateLundiSemaineCourante().get(Calendar.DAY_OF_MONTH));
		dateLundiWeakAfter.add(Calendar.DAY_OF_YEAR, 7);
		for(int i = 0; i<list.size(); i++) {
			if(list.get(i).getDateHeure().compareTo(controleurAgenda.getDateLundiSemaineCourante())>=0 
					&& list.get(i).getDateHeure().compareTo(dateLundiWeakAfter)<0) {				

				String heures = ""+list.get(i).getDateHeure().get(Calendar.HOUR_OF_DAY);
				if (list.get(i).getDateHeure().get(Calendar.HOUR_OF_DAY) < 10)
					heures = "0"+list.get(i).getDateHeure().get(Calendar.HOUR_OF_DAY);

				String minutes = ""+list.get(i).getDateHeure().get(Calendar.MINUTE);
				if (list.get(i).getDateHeure().get(Calendar.MINUTE) < 10)
					minutes = "0"+list.get(i).getDateHeure().get(Calendar.MINUTE);

				String lieu = "";
				if (list.get(i).getLieu() != null)
					lieu = " ("+list.get(i).getLieu()+")";

				String contactName = "";
				if(this.controleurAgenda.getIdContactFiltre() == null)
					contactName = this.controleurAgenda.getContactNameById(list.get(i).getIdcontact())+" - ";

				String strEvenenement = heures+"h"+minutes+": "+contactName+list.get(i).getObjet()+lieu;

				int jourSemaine = list.get(i).getDateHeure().get(Calendar.DAY_OF_WEEK);
				switch (jourSemaine){
				case Calendar.MONDAY:
					lundi.add(strEvenenement);
					break;
				case Calendar.TUESDAY:
					mardi.add(strEvenenement);
					break;
				case Calendar.WEDNESDAY:
					mercredi.add(strEvenenement);
					break;
				case Calendar.THURSDAY:
					jeudi.add(strEvenenement);
					break;			
				case Calendar.FRIDAY:
					vendredi.add(strEvenenement);
					break;
				case Calendar.SATURDAY:
					samedi.add(strEvenenement);
					break;			
				case Calendar.SUNDAY:
					dimanche.add(strEvenenement);
					break;			
				}				
			}
		}
		this.tabList[0].setListData(lundi.toArray());
		this.tabList[1].setListData(mardi.toArray());
		this.tabList[2].setListData(mercredi.toArray());
		this.tabList[3].setListData(jeudi.toArray());
		this.tabList[4].setListData(vendredi.toArray());
		this.tabList[5].setListData(samedi.toArray());
		this.tabList[6].setListData(dimanche.toArray());

		this.clearTextEvenement();
	}

	/**
	 * Methode utilisee dans les listeners (clavier et souris) pour afficher l'evenement selectionne
	 * @param e
	 */
	private void afficheEvenement(InputEvent e) {
		// si la JList d'où provient le click a un élément sélectionné
		if (((JList)e.getSource()).getSelectedIndex() != -1) {
			// MAJ de l'id de l'événement affiché
			int indiceEvenement = 0;
			for(int i = 0; i<tabList.length; i++){
				if(tabList[i].equals(((JList)e.getSource()))) {
					indiceEvenement += tabList[i].getSelectedIndex();
					break;
				}
				indiceEvenement += tabList[i].getModel().getSize();
			}
			controleurAgenda.setEvenementAffiche(controleurAgenda.getEvenements().get(indiceEvenement));		
			
			// on affiche cet événement
			String tmp = ((JList)e.getSource()).getSelectedValue().toString();
			// si il y a un filtre sur le contact
			if(this.controleurAgenda.getIdContactFiltre() != null) {
				tmp = tmp.replace(": ", "</b><br/>");
			}
			else {
				tmp = tmp.replace(" - ", "</b><br/>");
				tmp = tmp.replace(": ", " - ");
			}
			String duree = "";
			if(controleurAgenda.getEvenementAffiche().getDuree() != null)
				duree = ", <b>durée:</b> "+controleurAgenda.getEvenementAffiche().getDuree();
			textEvenement.setText("<html><b>"+tmp+duree+"</html>");							
		}
		else {
			clearTextEvenement();
		}
		for(int i = 0; i<tabList.length; i++){
			if(!tabList[i].equals((JList)e.getSource())) {
				tabList[i].clearSelection();
			}
		}
	}

	/**
	 * Classe interne: Mouse listener des JList
	 * @author erik helsens
	 */
	private class AfficheEvenementMouseListener extends MouseAdapter{
		public void mousePressed(MouseEvent e) {
			afficheEvenement(e);
		}		
	}

	/**
	 * Classe interne: Key listener des JList
	 * @author erik helsens
	 */
	private class AfficheEvenementKeyListener extends KeyAdapter{
		public void keyReleased(KeyEvent arg0) {
			if(arg0.getKeyCode() == KeyEvent.VK_DOWN || arg0.getKeyCode() == KeyEvent.VK_UP 
					|| arg0.getKeyCode() == KeyEvent.VK_PAGE_UP || arg0.getKeyCode() == KeyEvent.VK_PAGE_DOWN
					|| arg0.getKeyCode() == KeyEvent.VK_BEGIN || arg0.getKeyCode() == KeyEvent.VK_END
					|| arg0.getKeyCode() == KeyEvent.VK_SPACE ) {
				afficheEvenement(arg0);
			}
		}		
	}

	/**
	 * Initialise la liste des contacts
	 */
	private void setContacts() {
		this.comboContacts.removeAllItems();
		this.contacts = controleurAgenda.getControleur().getVue().getPanelRepertoire().getControleurRepertoire().getContacts();
		this.comboContacts.addItem("TOUS");
		for(int i=0; i<this.contacts.size(); i++){
			this.comboContacts.addItem(this.contacts.get(i).getNom().toUpperCase()+" "+this.contacts.get(i).getPrenom());
		}
	}


	public ControleurAgenda getControleurAgenda() {
		return controleurAgenda;
	}



	public void setControleurAgenda(ControleurAgenda controleurAgenda) {
		this.controleurAgenda = controleurAgenda;
	}



	public VueEvenement getVueEvenement() {
		return vueEvenement;
	}


	public JComboBox getComboContacts() {
		return comboContacts;
	}



	public ListeContact getContacts() {
		return contacts;
	}
}
