package vues;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.IOException;
import java.util.ArrayList;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextField;
import javax.swing.JTextPane;

import modeles.ListeContact;

import org.xml.sax.SAXException;
import org.xmldb.api.base.XMLDBException;

import controleurs.Controleur;
import controleurs.ControleurRepertoire;


/**
 * Vue associee au repertoire
 * @author erik helsens 
 */
public class PanelRepertoire extends JPanel {
	private static final long serialVersionUID = 2802754784871538224L;

	/**
	 * le controleur du repertoire
	 */
	private ControleurRepertoire controleurRepertoire;
	/**
	 * la vue de type VueContact permet d'ajouter ou d'editer un contact
	 */
	private VueContact vueContact;
	/**
	 * liste de contacts du repertoire
	 */
	private JList listeContacts;
	/**
	 * composant ou s'affiche les details sur un contact
	 */
	private JTextPane textContacts;
	/**
	 * champ de recherche du nom
	 */
	private JTextField textNom;
	/**
	 * champ de recherche du prenom
	 */
	private JTextField textPrenom;

	/**
	 * Constructeur
	 * @param c le controleur du programme
	 * @throws XMLDBException
	 * @throws SAXException
	 * @throws IOException
	 */
	public PanelRepertoire(Controleur c) throws XMLDBException, SAXException, IOException {
		// création du controleur du repertoire à partir du controleur du programme
		this.controleurRepertoire = new ControleurRepertoire(this,c.getService(),c.getCollection());

		// liste des contacts
		this.listeContacts = new JList();
		// listener associé à la souris
		this.listeContacts.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent e) {
				afficheContact();
			}			
		});
		// listener associé au clavier
		this.listeContacts.addKeyListener(new KeyAdapter(){
			public void keyReleased(KeyEvent arg0) {
				if(arg0.getKeyCode() == KeyEvent.VK_DOWN || arg0.getKeyCode() == KeyEvent.VK_UP 
						|| arg0.getKeyCode() == KeyEvent.VK_PAGE_UP || arg0.getKeyCode() == KeyEvent.VK_PAGE_DOWN
						|| arg0.getKeyCode() == KeyEvent.VK_BEGIN || arg0.getKeyCode() == KeyEvent.VK_END) {
					afficheContact();
				}
			}		
		});
		JScrollPane scrollListeContacts = new JScrollPane(this.listeContacts);
		
		/** panel de la recherche de contact **/
		// champs de recherche
		JPanel panelRechercheChamps = new JPanel(new GridLayout(4,1));
		textNom = new JTextField();
		textPrenom = new JTextField();
		panelRechercheChamps.add(new JLabel("Nom:"));
		panelRechercheChamps.add(textNom);
		panelRechercheChamps.add(new JLabel("Prénom:"));
		panelRechercheChamps.add(textPrenom);
		
		// boutton rechercher
		JPanel panelRechercheValidation = new JPanel(new FlowLayout(FlowLayout.RIGHT));
		JButton bRechercher = new JButton("Rechercher");
		bRechercher.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				try {
					controleurRepertoire.searchContacts(textNom.getText().trim(), textPrenom.getText().trim());
				} catch (XMLDBException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				} catch (SAXException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				} catch (IOException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
			}			
		});
		panelRechercheValidation.add(bRechercher);
		
		// panel de recherche
		JPanel panelRecherche = new JPanel(new BorderLayout());
		panelRecherche.add(panelRechercheChamps,BorderLayout.CENTER);
		panelRecherche.add(panelRechercheValidation,BorderLayout.SOUTH);
		
		
		// panel de gauche comportant la liste de contacts + les champs de recherche
		JPanel panelGauche = new JPanel(new BorderLayout());
		panelGauche.add(scrollListeContacts,BorderLayout.CENTER);
		panelGauche.add(panelRecherche,BorderLayout.SOUTH);
		
		

		// texte d'affichage des informations d'un contact	
		this.textContacts = new JTextPane();
		this.textContacts.setEditable(false);
		this.textContacts.setContentType("text/html");		
		JScrollPane scrollTextContacts = new JScrollPane(this.textContacts);

		// jointure des deux dans un JSplitPane
		JSplitPane panelSplit = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,panelGauche,scrollTextContacts);
		panelSplit.resetToPreferredSizes();
		panelSplit.setOneTouchExpandable(true);
		panelSplit.setDividerLocation(150);

		// Boutton Actualiser
		JButton bActualiser = new JButton("Actualiser");
		bActualiser.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				try {
					controleurRepertoire.requestAllContacts();
					clearTextContacts();
				} catch (XMLDBException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				} catch (SAXException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				} catch (IOException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}				
			}			
		});

		// Boutton Ajouter
		JButton bAjouter = new JButton("Ajouter");
		bAjouter.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				vueContact = new VueContact(controleurRepertoire,true,"");				
			}			
		});

		// Boutton Editer
		JButton bEditer = new JButton("Editer");
		bEditer.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				if (listeContacts.getSelectedIndex() != -1) {
					String id = controleurRepertoire.getContacts().get(listeContacts.getSelectedIndex()).getId();
					vueContact = new VueContact(controleurRepertoire,false, id);
					vueContact.setContact(controleurRepertoire.getContactAffiche());
				}
			}			
		});

		// Boutton Supprimer
		JButton bSupprimer = new JButton("Supprimer");
		bSupprimer.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				if (listeContacts.getSelectedIndex() != -1) {
					if (JOptionPane.showConfirmDialog(null, "Êtes-vous sur de vouloir supprimer ce contact?", 
							"Suppression d'un contact...", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE) 
							== JOptionPane.YES_OPTION) {
						try {
							if(!controleurRepertoire.removeContact(controleurRepertoire.getContacts().get(listeContacts.getSelectedIndex()).getId()))
								JOptionPane.showMessageDialog(null, 
										"Suppression impossible du contact car il possède\ndes entrées dans l'agenda !",
										"Suppression impossible du contact !", JOptionPane.ERROR_MESSAGE);
						} catch (XMLDBException e1) {
							// TODO Auto-generated catch block
							e1.printStackTrace();
						} catch (SAXException e1) {
							// TODO Auto-generated catch block
							e1.printStackTrace();
						} catch (IOException e1) {
							// TODO Auto-generated catch block
							e1.printStackTrace();
						}
					}
				}
			}			
		});

		// panel des bouttons
		JPanel panelBouttons = new JPanel(new GridLayout(1,4));
		panelBouttons.add(bActualiser);
		panelBouttons.add(bAjouter);
		panelBouttons.add(bEditer);
		panelBouttons.add(bSupprimer);

		// ajout des différents panels
		this.setLayout(new BorderLayout());
		this.add(panelBouttons,BorderLayout.SOUTH);
		this.add(panelSplit,BorderLayout.CENTER);

		// recherche effectuée sur tous les contacts
		this.controleurRepertoire.requestAllContacts();
	}

	/**
	 * Methode d'affichage du contact selectionne
	 */
	private void afficheContact() {
		if (listeContacts.getSelectedIndex() != -1) {
			try {
				// recherche sur un contact
				controleurRepertoire.requestOneContact(controleurRepertoire.getContacts().get(listeContacts.getSelectedIndex()).getId());
			} catch (XMLDBException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			} catch (SAXException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			} catch (IOException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}					
		}
	}
	

	/***************************************************************************************************************/
	// getters and setters
	public void setListe(ArrayList<Object> al) {		
		this.listeContacts.setListData(al.toArray());
	}

	public void clearTextContacts() {
		this.textContacts.setText("");
		this.controleurRepertoire.setContactAffiche(null);
	}

	public void setListe(ListeContact lc) {		
		ArrayList<String> listeTmp = new ArrayList<String>();
		for(int i = 0; i<lc.size(); i++){
			listeTmp.add(lc.get(i).getNom().toUpperCase()+" "+lc.get(i).getPrenom());
		}
		this.listeContacts.setListData(listeTmp.toArray());

		this.clearTextContacts();
	}

	public ControleurRepertoire getControleurRepertoire() {
		return controleurRepertoire;
	}

	public void setTextContacts(String text) {
		this.textContacts.setText(text);

	}

	public VueContact getVueContact() {
		return vueContact;
	}	

}
