package vues;
import java.io.IOException;

import javax.swing.JFrame;
import javax.swing.JTabbedPane;

import org.xml.sax.SAXException;
import org.xmldb.api.base.XMLDBException;

import controleurs.Controleur;


/**
 * Classe representant la vue du programme
 * @author erik helsens
 */
public class Vue {
	/**
	 * le controleur du programme
	 */
	private Controleur controleur;	
	/**
	 * Panel du repertoire
	 */
	private PanelRepertoire panelRepertoire;
	/**
	 * Panel de l'agenda
	 */
	private PanelAgenda panelAgenda;

	/**
	 * Contructeur de la vue
	 * @throws ClassNotFoundException
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 * @throws XMLDBException
	 * @throws IOException 
	 * @throws SAXException 
	 */
	public Vue(String nomCollection) throws ClassNotFoundException, InstantiationException, IllegalAccessException, XMLDBException, SAXException, IOException {
		// affectation du controleur
		this.controleur = new Controleur(this,nomCollection);		

		// panel principal du répertoire
		this.panelRepertoire = new PanelRepertoire(this.controleur);

		// panel principal de l'agenda
		this.panelAgenda = new PanelAgenda(this.controleur);

		// panel principal de l'application
		JTabbedPane tabbedPane = new JTabbedPane(JTabbedPane.TOP);		
		tabbedPane.add("REPERTOIRE", panelRepertoire);
		tabbedPane.add("AGENDA", panelAgenda);

		// fenetre
		JFrame fenetre = new JFrame("Projet XML  -  Erik HELSENS  [M1 MIAGE]");
		fenetre.setContentPane(tabbedPane);
		fenetre.setSize(700,500);
		fenetre.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		fenetre.setLocation(100, 100);
		fenetre.setVisible(true);
	}
	
	/***************************************************************************************************************/
	// getters and setters
	public Controleur getControleur() {
		return controleur;
	}

	public PanelRepertoire getPanelRepertoire() {
		return panelRepertoire;
	}

	public PanelAgenda getPanelAgenda() {
		return panelAgenda;
	}	
}
