package vues;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

import modeles.ContactFull;

import org.xml.sax.SAXException;
import org.xmldb.api.base.XMLDBException;

import controleurs.ControleurRepertoire;


/**
 * Vue utilisee pour l'ajout ou l'edition de contact dans le repertoire
 * @author erik helsens
 */
public class VueContact {
	private ControleurRepertoire controleurRepertoire;
	private JFrame fenetre;
	private JTextField textNom;
	private JTextField textPrenom;
	private JTextField textRue;
	private JTextField textCodepostal;
	private JTextField textVille;
	private JTextField textPays;
	private JTextField textTelephone1;
	private JTextField textTelephone2;
	private JTextField textTelephone3;
	private JTextField textEmail1;
	private JTextField textEmail2;
	private JComboBox comboTelephone1;
	private JComboBox comboTelephone2;
	private JComboBox comboTelephone3;

	/**
	 * Constructeur de la VueContact
	 * @param c le controleur du programme
	 * @param add un booleen qui est vrai si on est dans un cas d'ajout, faux si on est dans un cas d'edition
	 * @param id l'id du contact utilise en cas d'edition
	 */
	public VueContact(ControleurRepertoire c, final boolean add, final String id){
		this.controleurRepertoire = c;

		// initialisation des text fields et combo box
		this.textNom = new JTextField();
		this.textPrenom = new JTextField();
		this.textRue = new JTextField();
		this.textCodepostal = new JTextField();
		this.textVille = new JTextField();
		this.textPays = new JTextField();
		this.textTelephone1 = new JTextField();
		this.textTelephone2= new JTextField();
		this.textTelephone3= new JTextField();
		this.textEmail1 = new JTextField();
		this.textEmail2 = new JTextField();
		this.comboTelephone1 = new JComboBox(new String[]{"mobile","fixe","travail","autre"});
		this.comboTelephone2 = new JComboBox(new String[]{"mobile","fixe","travail","autre"});
		this.comboTelephone3 = new JComboBox(new String[]{"mobile","fixe","travail","autre"});
		
		// panels des numéros de téléphone
		JPanel panelTelephone1 = new JPanel(new GridLayout(1,3));
		panelTelephone1.add(this.textTelephone1);
		panelTelephone1.add(new JLabel(""));
		panelTelephone1.add(this.comboTelephone1);
		JPanel panelTelephone2 = new JPanel(new GridLayout(1,3));
		panelTelephone2.add(this.textTelephone2);
		panelTelephone2.add(new JLabel(""));
		panelTelephone2.add(this.comboTelephone2);
		JPanel panelTelephone3 = new JPanel(new GridLayout(1,3));
		panelTelephone3.add(this.textTelephone3);
		panelTelephone3.add(new JLabel(""));
		panelTelephone3.add(this.comboTelephone3);
				
		// panel des champs à remplir
		JPanel panelLibelles = new JPanel(new GridLayout(22,1));
		panelLibelles.add(new JLabel("Nom:"));
		panelLibelles.add(textNom);
		panelLibelles.add(new JLabel("Prénom:"));
		panelLibelles.add(textPrenom);
		panelLibelles.add(new JLabel("Rue:"));
		panelLibelles.add(textRue);		
		panelLibelles.add(new JLabel("Code Postal:"));
		panelLibelles.add(textCodepostal);		
		panelLibelles.add(new JLabel("Ville:"));
		panelLibelles.add(textVille);		
		panelLibelles.add(new JLabel("Pays:"));
		panelLibelles.add(textPays);		
		panelLibelles.add(new JLabel("Téléphone:"));
		panelLibelles.add(panelTelephone1);		
		panelLibelles.add(new JLabel("Téléphone:"));
		panelLibelles.add(panelTelephone2);		
		panelLibelles.add(new JLabel("Téléphone:"));
		panelLibelles.add(panelTelephone3);		
		panelLibelles.add(new JLabel("Email 1:"));
		panelLibelles.add(textEmail1);		
		panelLibelles.add(new JLabel("Email 2:"));
		panelLibelles.add(textEmail2);		

		// boutton enregistrer
		JButton bEnregistrer = new JButton("Enregister");
		bEnregistrer.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				// si le nom ou le prénom ne sont pas remplis
				if (textNom.getText().trim().equals("") || textPrenom.getText().trim().equals("")) {
					JOptionPane.showMessageDialog(fenetre, "Il faut obligatoirement remplir le nom et le prénom !" , "Attention !", JOptionPane.WARNING_MESSAGE);
				}
				else {
					ContactFull c;
					// en cas d'ajout
					if (add) {
						c = new ContactFull();
					}
					else {
						c = new ContactFull(id);
					}
					c.setNom(textNom.getText());
					c.setPrenom(textPrenom.getText());
					if (!textRue.getText().trim().equals("")) c.setRue(textRue.getText().trim());
					if (!textCodepostal.getText().trim().equals("")) c.setCodepostal(textCodepostal.getText().trim());
					if (!textVille.getText().trim().equals("")) c.setVille(textVille.getText().trim());
					if (!textPays.getText().trim().equals("")) c.setPays(textPays.getText().trim());
					if (!textTelephone1.getText().trim().equals("")) c.addNumero((String)comboTelephone1.getSelectedItem(), textTelephone1.getText().trim());
					if (!textTelephone2.getText().trim().equals("")) c.addNumero((String)comboTelephone2.getSelectedItem(), textTelephone2.getText().trim());
					if (!textTelephone3.getText().trim().equals("")) c.addNumero((String)comboTelephone3.getSelectedItem(), textTelephone3.getText().trim());
					if (!textEmail1.getText().trim().equals("")) c.addEmail(textEmail1.getText().trim());
					if (!textEmail2.getText().trim().equals("")) c.addEmail(textEmail2.getText().trim());
					try {
						// en cas d'ajout
						if (add) {
							controleurRepertoire.addContact(c);
						}
						// en cas d'édition
						else {
							controleurRepertoire.editContact(c);
						}
					} catch (XMLDBException e1) {
						// TODO Auto-generated catch block
						e1.printStackTrace();
					} catch (SAXException e1) {
						// TODO Auto-generated catch block
						e1.printStackTrace();
					} catch (IOException e1) {
						// TODO Auto-generated catch block
						e1.printStackTrace();
					}
					fenetre.setVisible(false);
					fenetre = null;
				}
			}			
		});

		// boutton annuler (ferme la fenêtre)
		JButton bAnnuler = new JButton("Annuler");
		bAnnuler.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {				
				fenetre.setVisible(false);
				fenetre = null;
			}			
		});

		// panel des bouttons
		JPanel panelBouttons = new JPanel(new GridLayout(1,3));
		panelBouttons.add(bAnnuler);
		panelBouttons.add(new JLabel());
		panelBouttons.add(bEnregistrer);

		// panel principal
		JPanel principal = new JPanel(new BorderLayout());
		principal.add(panelLibelles,BorderLayout.CENTER);
		principal.add(panelBouttons,BorderLayout.SOUTH);

		// fenêtre
		if (add)
			this.fenetre = new JFrame("Ajout d'un contact");
		else
			this.fenetre = new JFrame("Edition d'un contact");
		fenetre.setContentPane(principal);
		fenetre.setSize(300,500);
		fenetre.setLocation(100, 100);
		fenetre.setVisible(true);
	}
	
	/**
	 * Permet d'initialiser les informations du contact dans la vue d'edition
	 * @param c le contact
	 */
	public void setContact(ContactFull c) {
		textNom.setText(c.getNom());
		textPrenom.setText(c.getPrenom());
		if(c.getRue() != null) textRue.setText(c.getRue());
		if(c.getCodepostal() != null) textCodepostal.setText(c.getCodepostal());
		if(c.getVille() != null) textVille.setText(c.getVille());
		if(c.getPays() != null) textPays.setText(c.getPays());
		if(c.getTelephone().size()>0) {
			textTelephone1.setText(c.getTelephone().get(0).getNumero());
			comboTelephone1.setSelectedItem(c.getTelephone().get(0).getType());
			if(c.getTelephone().size()>1) {
				textTelephone2.setText(c.getTelephone().get(1).getNumero());
				comboTelephone2.setSelectedItem(c.getTelephone().get(1).getType());
			}
			if(c.getTelephone().size()>2) {
				textTelephone3.setText(c.getTelephone().get(2).getNumero());
				comboTelephone3.setSelectedItem(c.getTelephone().get(2).getType());
			}
		}
		if(c.getEmail().size()>0) {
			textEmail1.setText(c.getEmail().get(0));
			if(c.getEmail().size()>1)
				textEmail2.setText(c.getEmail().get(1));
		}		
	}
}
