package vues;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SpinnerDateModel;

import modeles.Evenement;
import modeles.ListeContact;

import org.xml.sax.SAXException;
import org.xmldb.api.base.XMLDBException;

import controleurs.ControleurAgenda;

/**
 * Vue utilisee pour l'ajout ou l'edition d'evenement dans l'agenda
 * @author erik helsens
 */
public class VueEvenement {
	private ControleurAgenda controleurAgenda;
	private JFrame fenetre;
	private JComboBox comboContacts;
	private JSpinner spinnerDate;
	private JComboBox comboHeures;
	private JComboBox comboMinutes;
	private JTextField textDuree;
	private JTextField textObjet;
	private JTextField textLieu;

	private ListeContact contacts;

	/**
	 * Constructeur de la VueEvenement
	 * @param c le controleur du programme
	 * @param add un booleen qui est vrai si on est dans un cas d'ajout, faux si on est dans un cas d'edition
	 * @param id l'id de l'evenement utilise en cas d'edition
	 */
	public VueEvenement(ControleurAgenda c, final boolean add, final String id){
		this.controleurAgenda = c;

		// initialisation des champs
		this.comboContacts = new JComboBox();
		this.contacts = controleurAgenda.getControleur().getVue().getPanelRepertoire().getControleurRepertoire().getContacts();
		for(int i=0; i<this.contacts.size(); i++){
			this.comboContacts.addItem(this.contacts.get(i).getNom().toUpperCase()+" "+this.contacts.get(i).getPrenom());
		}

		SpinnerDateModel model = new SpinnerDateModel(Calendar.getInstance().getTime(),null,null,Calendar.DAY_OF_YEAR);
		this.spinnerDate = new JSpinner(model);
		this.spinnerDate.setEditor(new JSpinner.DateEditor(this.spinnerDate,"dd/MM/yyyy"));
		

		this.comboHeures = new JComboBox();
		for(int i=0; i<=23; i++){
			this.comboHeures.addItem(new Integer(i).toString());
		}		
		this.comboMinutes = new JComboBox();
		for(int i=0; i<=59; i++){
			String tmp = new Integer(i).toString();;
			if(i<10)
				tmp = "0"+tmp;
			this.comboMinutes.addItem(tmp);
		}

		this.textDuree = new JTextField();
		this.textObjet = new JTextField();
		this.textLieu = new JTextField();


		// panel heure
		JPanel panelHeure = new JPanel(new FlowLayout(FlowLayout.LEFT));
		panelHeure.add(this.comboHeures);
		panelHeure.add(new JLabel(" : "));
		panelHeure.add(this.comboMinutes);

		// panel des champs à remplir
		JPanel panelLibelles = new JPanel(new GridLayout(12,1));
		panelLibelles.add(new JLabel("Contact concerné par l'événément:"));
		panelLibelles.add(this.comboContacts);
		panelLibelles.add(new JLabel("Date:"));
		panelLibelles.add(this.spinnerDate);
		panelLibelles.add(new JLabel("heure:"));
		panelLibelles.add(panelHeure);
		panelLibelles.add(new JLabel("Durée:"));
		panelLibelles.add(this.textDuree);
		panelLibelles.add(new JLabel("Objet:"));
		panelLibelles.add(this.textObjet);
		panelLibelles.add(new JLabel("Lieu:"));
		panelLibelles.add(this.textLieu);


		// boutton enregistrer
		JButton bEnregistrer = new JButton("Enregister");
		bEnregistrer.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				// si l'objet n'est pas rempli
				if (textObjet.getText().trim().equals("") ) {
					JOptionPane.showMessageDialog(fenetre, "Il faut obligatoirement remplir l'objet de l'événement !" , "Attention !", JOptionPane.WARNING_MESSAGE);
				}
				else {
					Evenement c;
					// en cas d'ajout
					if (add) {
						c = new Evenement();
					}
					else {
						c = new Evenement(id);
					}

					c.setIdcontact(contacts.get(comboContacts.getSelectedIndex()).getId());
					
					GregorianCalendar gc = new GregorianCalendar(); 
					gc.setTime((Date)spinnerDate.getModel().getValue());
					gc.set(Calendar.HOUR_OF_DAY, Integer.parseInt((String)comboHeures.getSelectedItem()));
					gc.set(Calendar.MINUTE, Integer.parseInt((String)comboMinutes.getSelectedItem()));
					c.setDateHeure(gc);
					
					if (!textDuree.getText().trim().equals("")) c.setDuree(textDuree.getText().trim());
					c.setObjet(textObjet.getText());
					if (!textLieu.getText().trim().equals("")) c.setLieu(textLieu.getText().trim());

					try {
						// en cas d'ajout
						if (add) {
							controleurAgenda.addEvenement(c);
						}
						// en cas d'édition
						else {
							controleurAgenda.editEvenement(c);
						}
					} catch (XMLDBException e1) {
						// TODO Auto-generated catch block
						e1.printStackTrace();
					} catch (SAXException e1) {
						// TODO Auto-generated catch block
						e1.printStackTrace();
					} catch (IOException e1) {
						// TODO Auto-generated catch block
						e1.printStackTrace();
					}
					fenetre.setVisible(false);
					fenetre = null;
				}
			}			
		});

		// boutton annuler (ferme la fenêtre)
		JButton bAnnuler = new JButton("Annuler");
		bAnnuler.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {				
				fenetre.setVisible(false);
				fenetre = null;
			}			
		});

		// panel des bouttons
		JPanel panelBouttons = new JPanel(new GridLayout(1,3));
		panelBouttons.add(bAnnuler);
		panelBouttons.add(new JLabel());
		panelBouttons.add(bEnregistrer);

		// panel principal
		JPanel principal = new JPanel(new BorderLayout());
		principal.add(panelLibelles,BorderLayout.CENTER);
		principal.add(panelBouttons,BorderLayout.SOUTH);

		// fenêtre
		if (add)
			this.fenetre = new JFrame("Ajout d'un événement");
		else
			this.fenetre = new JFrame("Edition d'un événement");
		fenetre.setContentPane(principal);
		fenetre.setSize(300,400);
		fenetre.setLocation(100, 100);
		fenetre.setVisible(true);
	}
	
	
	/**
	 * Permet d'initialiser les informations de l'evenement dans la vue d'edition
	 * @param c le contact
	 */
	public void setEvenement(Evenement c) {
		for(int i=0; i<this.contacts.size(); i++){
			if(this.contacts.get(i).getId().compareTo(c.getIdcontact()) == 0) {
				this.comboContacts.setSelectedIndex(i);
				break;
			}
		}
		this.spinnerDate.getModel().setValue(c.getDateHeure().getTime());
		this.comboHeures.setSelectedIndex(c.getDateHeure().get(Calendar.HOUR_OF_DAY));
		this.comboMinutes.setSelectedIndex(c.getDateHeure().get(Calendar.MINUTE));
		if(c.getDuree() != null) this.textDuree.setText(c.getDuree());
		this.textObjet.setText(c.getObjet());
		if(c.getLieu() != null) this.textLieu.setText(c.getLieu());
	}
}

